"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = promisePoller;

var _bluebird = _interopRequireDefault(require("bluebird"));

var _debug = _interopRequireDefault(require("debug"));

var _strategies = _interopRequireDefault(require("./strategies"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug.default)('promisePoller');
var DEFAULTS = {
  strategy: 'fixed-interval',
  retries: 5,
  shouldContinue: function shouldContinue(err) {
    return !!err;
  }
};
var pollerCount = 0;

function promisePoller() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  if (typeof options.taskFn !== 'function') {
    throw new Error('No taskFn function specified in options');
  }

  Object.keys(DEFAULTS).forEach(function (option) {
    return options[option] = options[option] || DEFAULTS[option];
  });
  options.name = options.name || "Poller-".concat(pollerCount++);
  debug("Creating a promise poller \"".concat(options.name, "\" with interval=").concat(options.interval, ", retries=").concat(options.retries));

  if (!_strategies.default[options.strategy]) {
    throw new Error("Invalid strategy \"".concat(options.strategy, "\". Valid strategies are ").concat(Object.keys(_strategies.default)));
  }

  var strategy = _strategies.default[options.strategy];
  debug("(".concat(options.name, ") Using strategy \"").concat(options.strategy, "\"."));
  var strategyDefaults = strategy.defaults;
  Object.keys(strategyDefaults).forEach(function (option) {
    return options[option] = options[option] || strategyDefaults[option];
  });
  debug("(".concat(options.name, ") Options:"));
  Object.keys(options).forEach(function (option) {
    debug("    \"".concat(option, "\": ").concat(options[option]));
  });
  return new _bluebird.default(function (resolve, reject) {
    var polling = true;
    var retriesRemaining = options.retries;
    var rejections = [];
    var timeoutId = null;

    if (options.masterTimeout) {
      debug("(".concat(options.name, ") Using master timeout of ").concat(options.masterTimeout, " ms."));
      timeoutId = setTimeout(function () {
        debug("(".concat(options.name, ") Master timeout reached. Rejecting master promise."));
        polling = false;
        reject('master timeout');
      }, options.masterTimeout);
    }

    function poll() {
      var task = options.taskFn();

      if (task === false) {
        task = _bluebird.default.reject('Cancelled');
        debug("(".concat(options.name, ") Task function returned false, canceling."));
        reject(rejections);
        polling = false;
      }

      var taskPromise = _bluebird.default.resolve(task);

      if (options.timeout) {
        taskPromise = taskPromise.timeout(options.timeout);
      }

      taskPromise.then(function (result) {
        debug("(".concat(options.name, ") Poll succeeded. Resolving master promise."));

        if (options.shouldContinue(null, result)) {
          debug("(".concat(options.name, ") shouldContinue returned true. Retrying."));
          var nextInterval = strategy.getNextInterval(options.retries - retriesRemaining, options);
          debug("(".concat(options.name, ") Waiting ").concat(nextInterval, "ms to try again."));

          _bluebird.default.delay(nextInterval).then(poll);
        } else {
          if (timeoutId !== null) {
            clearTimeout(timeoutId);
          }

          resolve(result);
        }
      }, function (err) {
        rejections.push(err);

        if (typeof options.progressCallback === 'function') {
          options.progressCallback(retriesRemaining, err);
        }

        if (! --retriesRemaining || !options.shouldContinue(err)) {
          debug("(".concat(options.name, ") Maximum retries reached. Rejecting master promise."));
          reject(rejections);
        } else if (polling) {
          debug("(".concat(options.name, ") Poll failed. ").concat(retriesRemaining, " retries remaining."));
          var nextInterval = strategy.getNextInterval(options.retries - retriesRemaining, options);
          debug("(".concat(options.name, ") Waiting ").concat(nextInterval, "ms to try again."));

          _bluebird.default.delay(nextInterval).then(poll);
        }
      });
    }

    poll();
  });
}